import pandas as pd
import numpy as np
import os
import h5py
import matplotlib.pyplot as plt
from einops import repeat
from . import source
from .source import filter_mag, read_filter
from .config import get_throughput
import pdb

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
refdata = os.path.join(path, 'refdata/')


def flux2electron(diameter, obscure, delta_lambda_per_pixel, spaxel_xsize, spaxel_ysize,
                  wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
    """

    Args:
        diameter：diameter of the telescope in cm
        obscure：obscure fraction of the telescope, value within [0, 1]
        spaxel_xsize: spaxel size in dispersion-axis in arcsec
        spaxel_ysize: spaxel size in spatial-axis in arcsec
        wavearr: wavelength range in Angstrom
        fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
        throughtputwave: wavelength array of the throughtput curve
        throughtputvalue: throughput fraction
        fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'

    Returns:

    """

    cc = 3.0e18   # speed of light, A/s
    spaxel_area = spaxel_xsize * spaxel_ysize    # arcsec^2
    d = diameter  # diameter of the telescope, in cm unit
    obscure = obscure  # effective central obscuration, no unit
    telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
    # delta_lambda_per_pixel = 1.755555  # per pixel
    delta_hz_per_specpixel = delta_lambda_per_pixel / wavearr**2 * cc  # s^-1
    specsampling = 1.0  # pixel
    planckh = 6.626e-27  # erg*s
    hv = planckh * cc / wavearr  # erg
    QE = 1

    if wavearr.ndim == 3:
        qvalue = np.zeros(shape=wavearr.shape, dtype=float)
        qvalue = np.interp(wavearr[:, 0, 0], throughtputwave, throughtputvalue)
        qvalue = repeat(qvalue, 'h -> h '+str(wavearr.shape[1])+' '+str(wavearr.shape[2]))
    elif wavearr.ndim == 1:
        qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)
    else:
        raise ValueError('check dimension of the input array.')


    if fluxunit=='erg/s/cm^2/A':
        isource1 = fluxarr * wavearr ** 2 / cc * spaxel_area  # erg/s/cm^2/Hz per spaxel
        isource2 = isource1 * telarea  # erg/s/Hz per spaxel
        isource3 = qvalue * isource2 * delta_hz_per_specpixel  # erg/s per spaxel
        isource4 = isource3 * specsampling  # erg/s per spec-element per spaxel
        isource5 = isource4 / hv  # phot/s per spec-element per spaxel
        isource6 = isource5 * QE  # e/s per spec-element per spaxel

        return isource6
    else:

        return np.nan


def electron2flux(config, wavearr, countsarr, throughtputwave, throughtputvalue, fluxunit=''):
    '''
    :param wavearr: A
    :param countsarr: e-/s
    :return:
    '''

    instru = config['configuration']

    cc = 3.0e18   # speed of light, A/s
    spaxel_area = instru['spaxel_xsize'] * instru['spaxel_ysize']    # arcsec^2
    d = 200  # diameter of the telescope, in cm unit
    obscure = 0.0  # effective central obscuration, no unit
    telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
    delta_lambda_per_pixel = 1.755555  # per pixel
    delta_hz_per_specpixel = delta_lambda_per_pixel / wavearr**2 * cc  # s^-1
    specsampling = 1.0  # 2 pixels
    planckh = 6.626e-27  # erg*s
    hv = planckh * cc / wavearr  # erg
    QE = 1

    qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)


    if fluxunit=='erg/s/cm^2/A':

        isource5 = countsarr / QE  # phot/s per spec-element per spaxel
        isource4 = isource5 * hv  # erg/s per spec-element per spaxel
        isource3 = isource4 / specsampling  # erg/s per spec-element per spaxel
        isource2 = isource3 / delta_hz_per_specpixel / qvalue  # erg/s per spaxel
        isource1 = isource2 / telarea  # erg/s/Hz per spaxel
        fluxarr = isource1 / wavearr ** 2 * cc / spaxel_area  # erg/s/cm^2/Hz per spaxel

        return fluxarr

    else:

        return np.nan



def get_nsky(wavearr):

    """

    Args:
        wavearr:
        spaxel_xsize:
        spaxel_ysize:

    Returns:
        sky counts (e-/s), in 1d array.


    """

    # know the sky counts in MS ccd per pixel in different bands,
    # Thus convert it to sky spectrum (counts) in IFS ccd per spaxel

    fluxskypp = np.zeros(len(wavearr))

    ii = np.logical_and(wavearr >= 2550, wavearr <= 4000)
    counta = len(np.where(ii == 1)[0])
    if counta > 0:
        fluxskypp[ii] = 0.028 / counta      # e/s/spaxel/pixel
    ii = np.logical_and(wavearr >= 4000, wavearr <= 6000)
    countb = len(np.where(ii == 1)[0])
    if countb > 0:
        fluxskypp[ii] = 0.229 / countb
    ii = np.logical_and(wavearr >= 6000, wavearr <= 9000)
    countc = len(np.where(ii == 1)[0])
    if countc > 0:
        fluxskypp[ii] = 0.301 / countc
    ii = np.where(wavearr >= 9000)[0]
    countd = len(ii)
    if countd > 0:
        fluxskypp[ii] = 0.301 / countd

    ms_pix_area = 0.074**2      # arcsec
    spaxel_area = 0.02**2
    factor = 0.9

    fluxskypp = fluxskypp / ms_pix_area * spaxel_area * factor  # e/s/spaxel/spec-elements

    return fluxskypp


def get_background(throughputfile, diameter, obscure, delta_lambda_per_pixel,
                   zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):

    """
    Consider zodiacal background and earth shine background in different levels.

    Args:
        zodia_level:
            zodiacal level, 'low', 'med', 'high', corresponding index from 1-3.
        earthshine_level:
            earth shine level, 'low', 'med', 'high', corresponding index from 1-3.
        wavearr:
            wave array to be returned.

    Returns:
        sky counts (e-/s) in each spaxel element (spaxel_xsize * spaxel_ysize), in 1d array.

    """

    # get IFS throughtput
    throughtputwave, throughtputvalue = get_throughput(throughputfile)

    # get the zodia spectrum, convert to e/s/spaxel/spec-elements
    filename = os.path.join(refdata, 'csst', 'background', 'zodi_spec.csv')
    cat = pd.read_csv(filename, comment='#')
    zodia_wave = cat['wave']
    zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
    zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
    zodia_counts_interp = flux2electron(diameter, obscure, delta_lambda_per_pixel,
                                        spaxel_xsize, spaxel_ysize,
                                       wavearr, zodia_spec_interp,
                                       throughtputwave, throughtputvalue,
                                       fluxunit='erg/s/cm^2/A')

    # get the earth shine spectrum, convert to e/s/spaxel/spec-elements
    filename = os.path.join(refdata, 'csst', 'background', 'earthshine_spec.csv')
    cat = pd.read_csv(filename, comment='#')
    earthshine_wave = cat['wave']
    earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
    earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
    earthshine_counts_interp = flux2electron(diameter, obscure, delta_lambda_per_pixel,
                                       spaxel_xsize, spaxel_ysize,
                                       wavearr, earthshine_spec_interp,
                                       throughtputwave, throughtputvalue,
                                       fluxunit='erg/s/cm^2/A')

    nsky = zodia_counts_interp + earthshine_counts_interp


    return nsky



class DetectorCountsRate(source.ModelCube):

    def __init__(self, config):

        self.config = config

        # pdb.set_trace()

        source.ModelCube.__init__(
            self, config
        )


        instru = config['configuration']

        throughtputwave, throughtputvalue = get_throughput(instru['efficiency_file'])

        self.source_counts = flux2electron(instru['diameter'],
                                               instru['obscure'],
                                               instru['wave_delta'],
                                               instru['spaxel_xsize'], instru['spaxel_ysize'],
                                               self.wavecube, self.fluxcube,
                                               throughtputwave, throughtputvalue,
                                               fluxunit='erg/s/cm^2/A')
        self.wavearr = self.wavecube



class DetectorCounts(object):

    def __init__(self, config, signal, obst=300, repn=20):

        instru = config['configuration']

        darkcube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['dark']      # e/s/pix

        readnoisecube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['readout_noise']   # e/pix

        # nskycube = get_nsky(signal.ccdspec_wave)        # e/s/specsampling
        nskycube = get_background(instru['efficiency_file'],
                                  instru['diameter'],
                                  instru['obscure'],
                                  instru['wave_delta'],
                                  config['zodiacal_level'],
                                   config['earthshine_level'],
                                   signal.ccdspec_wave,
                                   instru['spaxel_xsize'],
                                   instru['spaxel_ysize'])

        # repn = config['repn']
        # obst = config['obst']
        npixw = 2
        specsampling = 1

        tot_t = repn * obst
        readout_n = repn
        self.darkn = (darkcube * tot_t * npixw * specsampling)  # e/s/pix * sampling * t * extraction area
        self.rnn2 = readnoisecube**2 * (repn * npixw * specsampling)  # e/pix * sampling * readout times * extraction area
        self.sourcenn = (signal.source_counts * tot_t)  # e/s/sampling * t per spaxel
        self.skynn = (nskycube * tot_t)  # e/s/sampling * t per spaxel



class calculation_snr(object):

    def __init__(self, config, straylight=False):

        signal_rate = DetectorCountsRate(config)
        noise = DetectorCounts(config, signal_rate, obst=config['obst'], repn=config['repn'])

        self.source = noise.sourcenn
        self.darknoise = np.sqrt(noise.darkn)
        self.readnoise = np.sqrt(noise.rnn2)
        self.skynoise = np.sqrt(noise.skynn)
        self.sourcenoise = np.sqrt(noise.sourcenn)
        self.saturate_mask = (self.source > 65535)

        if straylight == False:
            self.straylight = np.zeros(shape=self.source.shape, dtype=float)
        else:
            # add 2% of the brightest source counts as straylight
            surface_brightness = np.sum(self.source, 0)
            ind_ycen, ind_xcen = np.where(surface_brightness == np.max(surface_brightness))
            nstraylight = self.source[:, ind_ycen[0], ind_xcen[0]] * 0.02

            self.straylight = nstraylight


        self.totnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
                                self.skynoise ** 2 + self.sourcenoise ** 2 +
                                self.straylight ** 2)
        self.sysnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
                                self.skynoise ** 2 + self.straylight ** 2)
        self.snr = noise.sourcenn / self.totnoise

        # model spectrum
        self.mockwave = signal_rate.wavearr
        self.mockflux = np.random.normal(signal_rate.fluxcube, scale=signal_rate.fluxcube / self.snr)
        self.mockerror = signal_rate.fluxcube / self.snr

        # model image
        self.img2d = signal_rate.mag2d

    def save(self, file):

        f = h5py.File(file, 'w')
        f.create_dataset('source', data=self.source)
        f.create_dataset('darknoise', data=self.darknoise)
        f.create_dataset('readnoise', data=self.readnoise)
        f.create_dataset('skynoise', data=self.skynoise)
        f.create_dataset('sourcenoise', data=self.sourcenoise)
        f.create_dataset('totnoise', data=self.totnoise)
        f.create_dataset('sysnoise', data=self.sysnoise)
        f.create_dataset('straylight', data=self.straylight)
        f.create_dataset('snr', data=self.snr)
        f.create_dataset('saturate_mask', data=self.saturate_mask)
        f.create_dataset('mockwave', data=self.mockwave)
        f.create_dataset('mockflux', data=self.mockflux)
        f.create_dataset('mockerror', data=self.mockerror)
        f.create_dataset('img2d', data=self.img2d)
        f.close()

    def plot(self, outfile):

        report = self

        fig, axs = plt.subplots(figsize=(7, 9))
        fig.subplots_adjust(hspace=0.35, wspace=0.45,
                            left=0.10, right=0.97, top=0.97, bottom=0.05)
        cm = plt.cm.get_cmap('jet_r')

        # ax_recontructed_magmap = plt.subplot2grid((3, 4), (0, 0))
        # im = ax_recontructed_magmap.imshow(self.img2d, origin='low', cmap=cm)
        # ax_recontructed_magmap.set_xlabel('spaxel')
        # ax_recontructed_magmap.set_ylabel('spaxel')
        # ax_recontructed_magmap.set_title('reconstructed magmap')
        # cbar_ax = fig.add_axes([0.25, 0.75, 0.01, 0.2])
        # cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(20, 25, 2), orientation='vertical')
        # cbar.set_label(r'mag/arcsec$^{2}$', labelpad=-1)
        # cbar.ax.tick_params(labelsize=7)

        # ax_recontructed_snr = plt.subplot2grid((3, 4), (1, 0))
        # ind = np.argmin(abs(self.mockwave - 5000))
        # im = ax_recontructed_snr.imshow(self.snr[ind, :, :], origin='low', cmap=cm)
        # ax_recontructed_snr.set_xlabel('spaxel')
        # ax_recontructed_snr.set_ylabel('spaxel')
        # ax_recontructed_snr.set_title('reconstructed SNR@5000A')
        # cbar_ax = fig.add_axes([0.25, 0.41, 0.01, 0.2])
        # cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(0, 4, 1), orientation='vertical')
        # cbar.set_label('SNR', labelpad=-1)
        # cbar.ax.tick_params(labelsize=7)

        # points = [[15, 15], [15, 20], [15, 25]]
        # npoints = len(points)

        # for k in range(npoints):

        k = 0
        # i = points[k][0]
        # j = points[k][1]

        ax_recontructed_spec = plt.subplot2grid((3, 3), (k, 0), colspan=3)
        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
        ax_recontructed_spec.set_ylabel('mock spec')
        ax_recontructed_spec.set_xlabel('wavelength')

        ax_snr = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
        ax_snr.plot(self.mockwave, self.snr, 'k-')
        ax_snr.set_ylabel('SNR')
        ax_snr.set_xlabel('wavelength')

        ax_noises = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
        ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
        ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
        ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
        ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
        ax_noises.legend(loc=1)
        ax_noises.set_ylabel('noise counts (e-)')

        plt.savefig(outfile)
        plt.close()


class read_report(object):

    def __init__(self, file):

        f = h5py.File(file, 'r')
        list(f.keys())

        self.source = f['source'].value
        self.darknoise = f['darknoise'].value
        self.readnoise = f['readnoise'].value
        self.skynoise = f['skynoise'].value
        self.sourcenoise = f['sourcenoise'].value
        self.totnoise = f['totnoise'].value
        self.sysnoise = f['sysnoise'].value
        self.straylight = f['straylight'].value
        self.snr = f['snr'].value
        self.saturate_mask = f['saturate_mask'].value
        self.mockwave = f['mockwave'].value
        self.mockflux = f['mockflux'].value
        self.mockerror = f['mockerror'].value
        self.img2d = f['img2d'].value
        f.close()


class calculation_limitmag(object):

    def __init__(self, config, straylight=False):

        instru = config['configuration']
        targetsnr = config['targetsnr']

        ccdspec_wave = np.arange(3500, 10000, 1.75555)
        darkc = instru['dark']      # e/s/pix
        rn = instru['readout_noise']   # e/pix
        nsky = get_nsky(ccdspec_wave)
        repn = config['repn']
        obst = config['obst']
        npixw = 2
        specsampling = 1

        coeff_s2 = (repn * obst) ** 2
        coeff_s1 = -targetsnr ** 2 * (repn * obst)
        coeff_s0 = -targetsnr ** 2 * (rn ** 2 * (repn * npixw * specsampling) +
                                (darkc * repn * obst * npixw * specsampling) +
                                (nsky * repn * obst))

        source_rate = (-coeff_s1 + np.sqrt(coeff_s1 ** 2 - 4 * coeff_s2 * coeff_s0)) / (2 * coeff_s2)

        throughtputwave, throughtputvalue = get_throughput(instru['efficiency_file'])
        flux = electron2flux(config, ccdspec_wave, source_rate,
                             throughtputwave, throughtputvalue,
                             fluxunit='erg/s/cm^2/A')

        filter = read_filter(config['source']['normalization']['band'])
        ind_filter = (ccdspec_wave >= filter.wavemin) & (ccdspec_wave <= filter.wavemax)
        filter_wave = ccdspec_wave[ind_filter]
        filter_flux = np.interp(filter_wave, filter.wave, filter.throughput)
        self.limitmag = filter_mag(ccdspec_wave, flux, filter_wave, filter_flux, output='mag')



class calculation_exptime(object):

    def __init__(self, config, straylight=False):

        instru = config['configuration']
        snr = config['targetsnr']

        ccdspec_wave = np.arange(3500, 10000, 1.75555)
        darkc = instru['dark']      # e/s/pix
        rn = instru['readout_noise']   # e/pix
        obst = config['obst']
        # nsky = get_nsky(ccdspec_wave)
        nsky = get_background(instru['efficiency_file'],
                              instru['diameter'],
                              instru['obscure'],
                              instru['wave_delta'],
                              config['zodiacal_level'],
                                  config['earthshine_level'],
                                  ccdspec_wave,
                                  instru['spaxel_xsize'],
                                  instru['spaxel_ysize'])

        npixw = 2
        specsampling = 1

        signal_rate = DetectorCountsRate(config)
        isource = signal_rate.source_counts

        # need to do loop, to limit the continous time per orbit and solve the repeat number.
        # in current situation, the exptime is larger than 300s, which is not relistic.
        repn = 1
        tmp_obst = obst * 2

        while tmp_obst / obst > 1.1:

            coeff_t2 = (isource * repn) ** 2
            coeff_t1 = -snr ** 2 * ((darkc * repn * npixw * specsampling) + (nsky * repn) + (isource * repn))
            coeff_t0 = -snr ** 2 * rn ** 2 * (repn * npixw * specsampling)

            obst_arr = (-coeff_t1 + np.sqrt(coeff_t1 ** 2 - 4 * coeff_t2 * coeff_t0)) / (2 * coeff_t2)

            filter = read_filter(config['source']['normalization']['band'])
            ind_filter = (ccdspec_wave >= filter.wavemin) & (ccdspec_wave <= filter.wavemax)
            tmp_obst = np.median(obst_arr[ind_filter])
            repn = tmp_obst * repn / obst

        self.exptime = tmp_obst * repn

        #--------
        noise = DetectorCounts(config, signal_rate, obst=config['obst'], repn=repn)

        self.source = noise.sourcenn
        self.darknoise = np.sqrt(noise.darkn)
        self.readnoise = np.sqrt(noise.rnn2)
        self.skynoise = np.sqrt(noise.skynn)
        self.sourcenoise = np.sqrt(noise.sourcenn)
        self.saturate_mask = (self.source > 65535)

        if straylight == False:
            self.straylight = np.zeros(shape=self.source.shape, dtype=float)
        else:
            # add 2% of the brightest source counts as straylight
            surface_brightness = np.sum(self.source, 0)
            ind_ycen, ind_xcen = np.where(surface_brightness == np.max(surface_brightness))
            nstraylight = self.source[:, ind_ycen[0], ind_xcen[0]] * 0.02

            self.straylight = nstraylight

        self.totnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
                                self.skynoise ** 2 + self.sourcenoise ** 2 +
                                self.straylight ** 2)
        self.sysnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
                                self.skynoise ** 2 + self.straylight ** 2)
        self.snr = noise.sourcenn / self.totnoise


        # model spectrum
        self.mockwave = signal_rate.wavearr
        self.mockflux = np.random.normal(signal_rate.fluxcube, scale=signal_rate.fluxcube / self.snr)
        self.mockerror = signal_rate.fluxcube / self.snr

        # model image
        self.img2d = signal_rate.mag2d


    def save(self, file):

        f = h5py.File(file, 'w')
        f.create_dataset('source', data=self.source)
        f.create_dataset('darknoise', data=self.darknoise)
        f.create_dataset('readnoise', data=self.readnoise)
        f.create_dataset('skynoise', data=self.skynoise)
        f.create_dataset('sourcenoise', data=self.sourcenoise)
        f.create_dataset('totnoise', data=self.totnoise)
        f.create_dataset('sysnoise', data=self.sysnoise)
        f.create_dataset('straylight', data=self.straylight)
        f.create_dataset('snr', data=self.snr)
        f.create_dataset('saturate_mask', data=self.saturate_mask)
        f.create_dataset('mockwave', data=self.mockwave)
        f.create_dataset('mockflux', data=self.mockflux)
        f.create_dataset('mockerror', data=self.mockerror)
        f.create_dataset('img2d', data=self.img2d)
        f.close()


    def plot(self, outfile):

        report = self

        fig, axs = plt.subplots(figsize=(7, 9))
        fig.subplots_adjust(hspace=0.35, wspace=0.45,
                            left=0.10, right=0.97, top=0.97, bottom=0.05)
        cm = plt.cm.get_cmap('jet_r')

        k = 0
        # i = points[k][0]
        # j = points[k][1]

        ax_recontructed_spec = plt.subplot2grid((3, 3), (k, 0), colspan=3)
        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
        ax_recontructed_spec.set_ylabel('mock spec')
        ax_recontructed_spec.set_xlabel('wavelength')

        ax_snr = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
        ax_snr.plot(self.mockwave, self.snr, 'k-')
        ax_snr.set_ylabel('SNR')
        ax_snr.set_xlabel('wavelength')

        ax_noises = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
        ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
        ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
        ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
        ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
        ax_noises.legend(loc=1)
        ax_noises.set_ylabel('noise counts (e-)')

        plt.savefig(outfile)
        plt.close()


def perform_calculation(config, calculation_mode='exptime2snr'):

    if calculation_mode == 'exptime2snr':

        report = calculation_snr(config)

    elif calculation_mode == 'snr2exptime':

        report = calculation_exptime(config)

    elif calculation_mode == 'snr2limitmag':

        report = calculation_limitmag(config)

    else:

        print('calculation_mode need to be " exptime2snr | snr2exptime | snr2limitmag "')
        report = None
        pdb.set_trace()


    return report


import numpy as np
import pandas as pd
import os
import astropy.io.fits as fits
from scipy.integrate import simps
import extinction
import pdb

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
refdata = os.path.join(path, 'refdata/')



class read_template(object):

    r"""
    get spectrum template using the input parameters

    .. todo::
        enable to add emission lines


    Args:
        isource (dict, including 3 keys):
            name:
                the spectrum filename
            redshift:
                the redshift applied to the template
            ebv:
                the extinction applied to the template

    Attributes:

        wave (`numpy.ndarray`, unit in Angstrom):
            the wavelength of the spectrum, redshift applied.
        flux (`numpy.ndarray`, unit in erg/s/cm^2/A)
            the flux of the spectrum, redshift and extinction considered.

    """

    def __init__(self, isource):

        # self.source = isource
        # self.line_definitions = self.source['lines']

        # snpp_path = '/Users/linlin/Pro/snpp_v1_05/refdata/'
        # template_filename = snpp_refdata + 'sed/' + self.source['name']
        # filtera = snpp_path + 'normalization/filters/sdss_g0.par'
        # result = input_mag_model(self.source['normalization']['value'],
        #                          galtpl, filtera)  # scale the template to given brightness

        template_filename = isource['name']
        template_redshift = isource['redshift']
        template_ebv = isource['ebv']

        if template_filename.endswith('.fits'):

            if template_filename in ['SFgal_texp_FeH0_tau5_Ew10_AGN1.fits',
                                     'SFgal_texp_FeH0_tau1_Ewd.fits',
                                     'SFgal_texp_FeH0_tau5_Ew5.fits',
                                     'SFgal_texp_FeH-2_tau10_Ew50.fits']:
                template_filename = os.path.join(refdata, 'sed', template_filename)

            hdu = fits.open(template_filename)
            template_wave = hdu[1].data['wavelength']   # A
            template_flux = hdu[1].data['flux'] * 1e-12         # erg/s/A/cm2

            # If the data is big endian, swap the byte order to make it little endian
            if template_wave.dtype.byteorder == '>':
                template_wave = template_wave.byteswap().newbyteorder()

        elif template_filename.endswith('.txt'):

            cat = pd.read_csv(template_filename, sep='\s+', header=None)
            template_wave = cat[0]      # unit should be in Angstrom
            template_flux = cat[1]      # unit should be in erg/s/cm^2/A

        else:
            raise ValueError("File name need to be ended with '.fits' or '.txt'. ")

        # extinction require the dtype should be double.
        template_wave = np.array(template_wave, dtype='float64')

        # extinction correction
        # assuming Calzetti-00 extinction law.
        r_v = 4.5
        extinction_mag = extinction.calzetti00(template_wave,
                                               template_ebv * r_v,
                                               r_v, unit='aa')
        template_flux = extinction.apply(extinction_mag, template_flux)

        # redshift correction
        template_wave = template_wave * (1 + template_redshift)
        template_flux = template_flux / (1 + template_redshift)

        # 检查输入flux是否有负值，避免噪声sqrt(n)之后会出现nan.
        ind = template_flux < 0
        template_flux[ind] = 0

        self.wave = template_wave
        self.flux = template_flux


def get_wavearr(config):

    instr_par = config['configuration']
    wave_start = instr_par['wave_start']
    wave_end = instr_par['wave_end']
    wave_delta = instr_par['ccd_xsize'] * instr_par['readout_xbin']

    ccdspec_wave = np.arange(wave_start, wave_end, wave_delta)

    return ccdspec_wave


class read_filter(object):

    def __init__(self, filter_name=''):
        """

        Args:
            filter_name:
        """

        filter0, filter1 = filter_name.split('_')
        if filter0 == 'sdss':
            filter_file = os.path.join(refdata, 'normalization', 'filters', filter_name+'.par')
            band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
            self.wave = band[0].values  # A
            self.throughput = band[1].values  # vaccum_pass, per photon
            self.wavemin = self.wave[0]
            self.wavemax = self.wave[-1]

        elif filter0 == 'Johnson':
            filter_file = os.path.join(refdata, 'normalization', 'filters', filter_name+'.txt')
            band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
            self.wave = (band[0].values)[::-1] * 10     # A
            self.throughput = (band[1].values)[::-1]    # no sure per photon, or per energy. (??)
            self.wavemin = self.wave[0]
            self.wavemax = self.wave[-1]

        elif filter0 == 'GALEX':
            filter_file = os.path.join(refdata, 'normalization', 'filters', filter_name+'.par')
            band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
            self.wave = band[0].values             # A
            self.throughput = band[1].values       # no sure per photon, or per energy. (??)
            self.wavemin = self.wave[0]
            self.wavemax = self.wave[-1]

        else:
            print('un_recognized filter system.')


        # find the central wavelength, effective wavelength, and FWHM of the given filter
        # filtermid = (self.wavemax - self.wavemin) * 0.5  # A, central wavelength
        dwave = self.wave[1:] - self.wave[:-1]
        self.waveeff = np.nansum(dwave * self.wave[1:] * self.throughput[1:]) / \
                       np.nansum(dwave * self.throughput[1:])
                                                                     # A, effective wavelength
        rmax = np.max(self.throughput)
        nnn = np.where(self.throughput > 0.5 * rmax)[0]
        self.FWHMmin = self.wave[nnn[0]]      # wave range at FWHM
        self.FWHMmax = self.wave[nnn[-1]]
        self.wavefwhm = self.FWHMmax - self.FWHMmin  # A, FWHM


def filter_mag(objwave, objflux, filterwave, filterthroughtput, output='mag'):
    """
    Get AB magnitude in a certain filter.

    Args:
        objwave: unit as A
        objflux: unit as erg/s/cm^2/A
        filterwave: unit as A
        filterthroughtput: unit as detector signal "per photon" (use vaccum_pass for space telescope,
                              otherwise select a given airmass)

    Return:
        AB magnitude in this band

    """

    # resample the throughtput to objwave
    ind = (objwave >= np.min(filterwave)) & (objwave <= np.max(filterwave))
    wavetmp = objwave[ind]
    phot_frac = np.interp(wavetmp, filterwave, filterthroughtput)  # phot fraction (/s/A/cm^2?)

    # convert to energy fraction
    # E_frac = E0/hv * N_frac / E0 ~ N_frac/nu ~ N_frac * lambda
    energy_frac = phot_frac * wavetmp  # convert to energy fraction

    # convert the objflux to erg/s/cm^2/Hz
    c = 3e18  # A/s
    objflux_hz = objflux[ind] * (objwave[ind] ** 2) / c  # erg/s/scm^2/Hz

    from scipy.integrate import simps
    integrate_flux = simps(objflux_hz * energy_frac, c / wavetmp) / simps(energy_frac, c / wavetmp)

    if output == 'mag':
        mag = -2.5 * np.log10(integrate_flux) - 48.6
        return mag
    elif output == 'flux':
        return integrate_flux
    else:
        print('Error: output need to be "mag" or "flux". ')
        return np.nan



def normalized(template_wave, template_flux, normalize_config):
    """

    Args:
        template_wave:
        template_flux:
        normalize_config: {'value', 'band'}

    Returns:

    """

    # source = config['source']
    # normalize = source['normalization']

    filter = read_filter(normalize_config['band'])

    ind_filter = (template_wave >= filter.wavemin) & (template_wave <= filter.wavemax)
    filter_wave = template_wave[ind_filter]
    filter_flux = np.interp(filter_wave, filter.wave, filter.throughput)
    filter_constant = simps(filter_flux * filter_wave, filter_wave)

    template_constant = simps(filter_flux * template_wave[ind_filter] * template_flux[ind_filter],
                              template_wave[ind_filter])

    u0 = normalize_config['value']
    u0 = 10**((u0 + 48.6)/(-2.5))         # target flux in erg/s/cm^2/Hz unit
    u0 = u0 * 3e18 / filter.waveeff**2    # erg/s/cm^2/A

    factor = u0 * filter_constant / template_constant

    norm_wave = template_wave                  # A
    norm_flux = template_flux * factor         # erg/s/cm^2/A


    # check the normalized magnitude should be normalized.value - 2.5*log(0.04)
    mag = filter_mag(norm_wave, norm_flux,
                     filter_wave, filter_flux, output='mag')


    return norm_wave, norm_flux, mag


class ModelCube(object):

    def __init__(self, config):

        self.ccdspec_wave = get_wavearr(config)
        self.ccdspec_nw = len(self.ccdspec_wave)


        # grid = get_grid(config)

        object = config['source']['spectrum']


        template = read_template(object)
        template_wave_interp = self.ccdspec_wave
        template_flux_interp = np.interp(template_wave_interp, template.wave, template.flux)

        # self.ccdspec_flux = np.zeros(shape=(self.ccdspec_nw, grid.ny, grid.nx), dtype=np.float32)

        self.wavecube, self.fluxcube, self.mag2d = normalized(template_wave_interp, template_flux_interp,
                                                              config['source']['normalization'])




import os
import numpy as np

from .config import get_instrument_config
import pdb

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
refdata_dir = os.path.join(path, 'refdata/')


def check_input(input_message='', input_type=None, input_range=(), input_default=None):

    # input_message:
    # input_type:
    # input_range:
    # input_default:

    tmp = False
    xx = None
    while tmp is False:

        xx_str = input(input_message + ' (default = ' + str(input_default) + '): ')

        if xx_str is '':

            xx = input_type(input_default)
            tmp = True

        else:

            try:
                xx = input_type(xx_str)

                if input_type is float or input_type is int:
                    flag_range = input_range[0] <= xx <= input_range[1]
                elif input_type is str:
                    flag_range = xx in input_range
                else:
                    flag_range = False

                if flag_range is True:
                    tmp = True
                else:
                    err_message = 'The input should be in ' + str(input_range)
                    print(err_message)
                    tmp = False

            except Exception:
                err_message = 'The input should be ' + str(input_type)
                print(err_message)
                tmp = False

    return xx


def build_input_para():

    calc = dict()

    print('######################################################')
    print('#          IFS Exposure Time Calculator              #')
    print('# (Use default value when press enter and no input)  #')
    print('######################################################')

    print('#')
    print('# 1. Spectral Resolution: 3.5 Angstrom per 2 pixels (fixed)')
    print('#    Spectral coverage: 3500-10000 A')

    print('#')
    print('# 2. CCD parameters: ')
    print('#     1. Dispersion axis binning: 1 pixel')
    print('#     2. Spatial axis binning: 1 pixel')
    print('#     3. Gain: 1 e-/ADU')
    print('#     4. Dark: 0.017 e-/s/pixel')
    print('#     5. readout_noise: 5.5 e-/pixel')

    readout_xbin = 1
    readout_ybin = 1
    gain = 1
    dark = 0.017
    readout_noise = 5.5

    tmp1 = False
    while tmp1 is False:

        print('# Whether or not change the default paramters: ')
        print('#    1. Reset the parameters.')
        print('#    2. Use the default paramters.')
        flag_change = check_input(input_message='Enter the index: ',
                                  input_type=int,
                                  input_range=(1, 2),
                                  input_default='2')

        if flag_change == 1:

            tmp2 = False
            while tmp2 is False:

                flag_number = input('The parameter index you want to change (1-5): ')
                if flag_number == '1':
                    readout_xbin = check_input(input_message='Dispersion axis binning',
                                input_type=int,
                                input_range=(1, 10),
                                input_default=1)

                    flag_next = input('whether or not change another one ? (y|n): ')
                    if flag_next == 'n':
                        tmp2 = True
                    else:
                        tmp2 = False

                elif flag_number == '2':
                    readout_ybin = check_input(input_message='Spatial axis binning',
                             input_type=int,
                             input_range=(1, 10),
                             input_default=1)

                    flag_next = input('whether or not change another one ? (y|n): ')
                    if flag_next == 'n':
                        tmp2 = True
                    else:
                        tmp2 = False

                elif flag_number == '3':
                    gain = check_input(input_message='Gain value',
                             input_type=float,
                             input_range=(1., 4.),
                             input_default=1)

                    flag_next = input('whether or not change another one ? (y|n): ')
                    if flag_next == 'n':
                        tmp2 = True
                    else:
                        tmp2 = False

                elif flag_number == '4':
                    dark = check_input(input_message='Dark value',
                             input_type=float,
                             input_range=(0., 0.1),
                             input_default=0.017)

                    flag_next = input('whether or not change another one ? (y|n): ')
                    if flag_next == 'n':
                        tmp2 = True
                    elif flag_next == 'y':
                        tmp2 = False
                    else:
                        print('The input needs to be (y|n). Retry.')
                        tmp2 = False

                elif flag_number == '5':
                    readout_noise = check_input(input_message='Readout Noise',
                                       input_type=float,
                                       input_range=(2, 7),
                                       input_default=5.5)

                    flag_next = input('whether or not change another one ? (y|n): ')
                    if flag_next == 'n':
                        tmp2 = True
                    elif flag_next == 'y':
                        tmp2 = False
                    else:
                        print('The input needs to be (y|n). Retry.')
                        tmp2 = False

                else:
                    print('The input needs to be (1-5). Retry.')
                    tmp2 = False

            tmp1 = True
            print('#')
            print('# 2. The reseted CCD parameters: ')
            print('#     1. Dispersion axis binning: ' + str(readout_xbin) + ' pixel')
            print('#     2. Spatial axis binning: ' + str(readout_ybin) + ' pixel')
            print('#     3. Gain: ' + str('{:3.1f}'.format(gain)) + ' e-/ADU')
            print('#     4. Dark: ' + str('{:4.3f}'.format(dark)) + ' e-/s/pixel')
            print('#     5. Readout noise: ' + str('{:4.3f}'.format(readout_noise)) + ' e-/pixel')

        elif flag_change == 2:

            print('# Use the default CCD parameters. ')
            tmp1 = True

    calc['configuration'] = get_instrument_config(readout_xbin=readout_xbin,
                                                  readout_ybin=readout_ybin,
                                                  readout_noise=readout_noise,
                                                  gain=gain,
                                                  dark=dark)

    # calculation modes.
    obst = 300.
    print('#')
    print('# 3. Specify the calculation mode: ')
    print('#     1. How many visits needed to obtain a certain SNR [under development, useless.].')
    print('#     2. SNR reached in certain visits.')
    print('#     (Set the exposure time = 300 seconds per visit.)')

    calc_mode = check_input(input_message='which calculation mode',
                             input_type=int,
                             input_range=(1, 2),
                             input_default=2)

    if calc_mode == 1:

        targetsnr = check_input(input_message='The target SNR is',
                             input_type=float,
                             input_range=(3, np.inf),
                             input_default=20)
        calc['calc_mode'] = {'mode': calc_mode, 'targetsnr': targetsnr, 'obst': obst}

        print('# Calculate the visits to obtain SNR = ' + str(targetsnr) + '. ')

    elif calc_mode == 2:

        repn = check_input(input_message='How many visits',
                             input_type=int,
                             input_range=(1, np.inf),
                             input_default=20)
        calc['calc_mode'] = {'mode': calc_mode, 'repn': repn, 'obst': obst}

        print('# Calculate the SNR in 300 * '+str(repn)+' seconds. ')





    print('#')
    print('# 4. Specify the source: ')
    print('#     1. import a simulated datacube. ')
    print('#     2. Define the source (position, shape, spectrum, brightness).')
    source_mode = check_input(input_message='Select one way',
                            input_type=int,
                            input_range=(1, 2),
                            input_default=2)
    calc['source_mode'] = source_mode

    # import the simulated datacube.
    if source_mode == 1:
        print('#')
        print('# 4. Upload a FITS file.')
        tmp = False
        datacube_file = ''
        while tmp is False:
            datacube_file = input('Enter the cube FITS file: ')
            file_exists = os.path.exists(datacube_file)
            if file_exists:
                tmp = True
            else:
                print('Do not found ' + datacube_file + ', retry.')
                tmp = False
        calc['source'] = {'filename': datacube_file}

    # define the source.
    elif source_mode == 2:

        scene = []
        print('#')
        print('# 4. Define the source (position, shape, spectrum, brightness). ')
        n_source = check_input(input_message='How many sources to be considered',
                                 input_type=int,
                                 input_range=(1, 10),
                                 input_default=1)

        print('# Set the parameters for each source.')
        for i in range(n_source):

            dic = dict()

            # set the source shape
            print('# 4. (1/5) Select the source type for source-' + str(i+1) + ': ')
            print('#    1. point source')
            print('#    2. extended source')
            isource_type = check_input(input_message='Enter the index',
                             input_type=int,
                             input_range=(1, 2),
                             input_default=2)

            # set the central xy-offset
            print('# 4. (2/5) Set the center position for source-' + str(i+1) + ': ')
            x_offset = check_input(input_message='Enter the x-offset [arcsec]',
                                         input_type=float,
                                         input_range=(-3, 3),
                                         input_default=0.0)
            y_offset = check_input(input_message='Enter the y-offset [arcsec]',
                                         input_type=float,
                                         input_range=(-3, 3),
                                         input_default=0.0)
            dic['position'] = {'x_offset': x_offset, 'y_offset': y_offset}


            if isource_type == 1:

                print('# 4. (3/5) Point source is set to have PSF shape with FWHM = 0.2". ')
                dic['geometry'] = {'shape': 'PSF', 'FWHM': 0.2}


            elif isource_type == 2:

                print('# 4. (3/5) Select the shape type for extended source: ')
                print('#    1. Gaussian')
                print('#    2. Sersic')
                print('#    3. flat_box')
                shape_type = check_input(input_message='Enter the index',
                                               input_type=int,
                                               input_range=(1, 3),
                                               input_default=2)

                if shape_type == 1:

                    fwhm = check_input(input_message='Enter the gaussian FWHM [arcsec]',
                                           input_type=float,
                                           input_range=(0.5, 6),
                                           input_default=1)

                    dic['geometry'] = {'shape': 'gaussian', 'FWHM': fwhm}


                elif shape_type == 2:

                    sersic_n = check_input(input_message='Enter the sersic index n',
                                           input_type=float,
                                           input_range=(0.5, 6),
                                           input_default=1)

                    re_maj = check_input(input_message='Enter the effective radius [arcsec]',
                                           input_type=float,
                                           input_range=(0, np.inf),
                                           input_default=2)

                    b2a = check_input(input_message='Enter the ratio of minor axis and major axis (b/a)',
                                           input_type=float,
                                           input_range=(0, 1),
                                           input_default=0.5)

                    pa = check_input(input_message='Enter position angle of major axis (North=0, East=90) [degree]',
                                           input_type=float,
                                           input_range=(0, 180),
                                           input_default=45)

                    dic['geometry'] = {'shape': 'sersic', 'sersic_n': sersic_n, 'Re_maj': re_maj,
                                       'PA': pa, 'b2a': b2a}


                elif shape_type == 3:

                    xwidth = check_input(input_message='Enter the width in x-axis [arcsec]',
                                           input_type=float,
                                           input_range=(1, 30),
                                           input_default=5.)

                    ywidth = check_input(input_message='Enter the width in y-axis [arcsec]',
                                           input_type=float,
                                           input_range=(1, 30),
                                           input_default=5.)

                    pa = check_input(input_message='Enter the rotation angle [degree]',
                                           input_type=float,
                                           input_range=(0, 90),
                                           input_default=45)

                    dic['geometry'] = {'shape': 'flatbox', 'PA': pa,
                                       'xwidth': xwidth, 'ywidth': ywidth}




            # choose or upload a spectrum template
            print('# 4. (4/5) Choose a spectrum for the source-' + str(i+1) + ': ')
            print('#    1. Elliptical galaxy (bc03_FeH0_tau1_Ewd)')
            print('#    2. Spiral galaxy (bc03_FeH0_tau5_Ew5)')
            print('#    3. Starburst galaxy (bc03_FeH-2_tau10_Ew50)')
            print('#    4. AGN (FeH0_tau5_Ew10_AGN)')
            print('#    5. upload spectrum file [text format only, '
                  'with two-columns: wavelength in A and flux in erg/s/cm^2/A]')
            template_index = check_input(input_message='Enter the choise',
                                         input_type=int,
                                         input_range=(1, 5),
                                         input_default=1)

            if template_index == 1:
                template_file = os.path.join(refdata_dir, 'sed', 'SFgal_texp_FeH0_tau1_Ewd.fits')
            elif template_index == 2:
                template_file = os.path.join(refdata_dir, 'sed', 'SFgal_texp_FeH0_tau5_Ew5.fits')
            elif template_index == 3:
                template_file = os.path.join(refdata_dir, 'sed', 'SFgal_texp_FeH-2_tau10_Ew50.fits')
            elif template_index == 4:
                template_file = os.path.join(refdata_dir, 'sed', 'SFgal_texp_FeH0_tau5_Ew10_AGN1.fits')
            else:
                template_file = None
                tmp = False
                while tmp is False:
                    template_file = input('Enter the spectrum file: ')
                    file_exists = os.path.exists(template_file)
                    if file_exists:
                        tmp = True
                    else:
                        print('Do not found ' + template_file + ', retry.')
                        tmp = False


            template_redshift = check_input(input_message='Enter the redshift',
                                            input_type=float,
                                            input_range=(0, np.inf),
                                            input_default=0.0)

            template_ebv = check_input(input_message='Enter the E(B-V)',
                                       input_type=float,
                                       input_range=(0, np.inf),
                                       input_default=0.0)

            dic['spectrum'] = {'name': template_file,
                               'redshift': template_redshift, 'ebv': template_ebv}


            # set the normalization
            print('# 4. (5/5) Select the normalization type for source-' + str(i+1) + ': ')
            print('#    1. Normalized at certain wavelength. ')
            print('#    2. Normalized at certain filter. ')
            inormalization_type = check_input(input_message='Select the normalization type',
                                              input_type=int,
                                              input_range=(1, 2),
                                              input_default=2)

            if inormalization_type == 1:

                normal_wave = check_input(input_message='Enter the wavelength [A]',
                                          input_type=float,
                                          input_range=(3500, 10000),
                                          input_default=5500)

                print('# Normalization unit: [add arcsec^-2 for extended source].')
                print('#    1. erg/s/cm^2/A')
                print('#    2. erg/s/cm^2/Hz')
                print('#    3. AB magnitude')
                normal_unit_flag = check_input(input_message='Select the normalization unit',
                                          input_type=int,
                                          input_range=(1, 3),
                                          input_default=1)

                if normal_unit_flag == 1:
                    normal_unit = 'erg/s/cm^2/A'
                elif normal_unit_flag == 2:
                    normal_unit = 'erg/s/cm^2/Hz'
                else:
                    normal_unit = 'AB magnitude'

                normal_value = check_input(input_message='Enter the normalization value',
                                          input_type=float,
                                          input_range=(0, np.inf),
                                          input_default=1e-17)

                dic['normalization'] = {'value': normal_value, 'unit': normal_unit, 'wavelength': normal_wave}


            else:

                print('# Filters: 1. sdss-u, 2. sdss-g, 3. sdss-r, 4. sdss-i, 5. sdss-z. ')
                normal_filter_index = check_input(input_message='Select the normalized filter',
                                          input_type=int,
                                          input_range=(1, 5),
                                          input_default=2)
                if normal_filter_index == 1:
                    normal_filter = 'sdss_u'
                elif normal_filter_index == 2:
                    normal_filter = 'sdss_g'
                elif normal_filter_index == 3:
                    normal_filter = 'sdss_r'
                elif normal_filter_index == 4:
                    normal_filter = 'sdss_i'
                else:
                    normal_filter = 'sdss_z'

                print('# Normalization unit: [add arcsec^-2 for extended source].')
                print('#    1. AB magnitude')
                print('#    2. erg/s/cm^2/A')
                print('#    3. erg/s/cm^2/Hz')

                normal_unit_flag = check_input(input_message='Select the normalization unit',
                                          input_type=int,
                                          input_range=(1, 3),
                                          input_default=1)

                if normal_unit_flag == 2:
                    normal_unit = 'erg/s/cm^2/A'
                elif normal_unit_flag == 3:
                    normal_unit = 'erg/s/cm^2/Hz'
                else:
                    normal_unit = 'AB magnitude'

                normal_value = check_input(input_message='Enter the normalization value',
                                          input_type=float,
                                          input_range=(0, np.inf),
                                          input_default=17.7)

                dic['normalization'] = {'value': normal_value, 'unit': normal_unit, 'band': normal_filter}


            # append to the scene one-by-one source.
            scene.append(dic)

        calc['source'] = scene


    # set the background level
    print('#')
    print('# 5. Specify the background level: ')
    print('# Zodiacal level: 1. low, 2. median, 3. high.')
    zodiacal_flag = check_input(input_message='Enter the choice',
                           input_type=int, input_range=(1, 3), input_default=2)
    if zodiacal_flag == 1:
        zodiacal = 'low'
    elif zodiacal_flag == 2:
        zodiacal = 'med'
    else:
        zodiacal = 'high'

    print('# Earth shine level: 1. low, 2. median, 3. high.')
    earth_shine_flag = check_input(input_message='Enter the choice',
                           input_type=int, input_range=(1, 3), input_default=2)
    if earth_shine_flag == 1:
        earth_shine = 'low'
    elif earth_shine_flag == 2:
        earth_shine = 'med'
    else:
        earth_shine = 'high'

    calc['background'] = {'zodiacal': zodiacal, 'earth_shine': earth_shine}



    return calc


if __name__ == '__main__':

    config = build_input_para()





import os
import pandas as pd
import json

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
refdata = os.path.join(path, 'refdata/')


# def get_telescope_config(diameter=200, obscure=0):
#
#     dict = {}
#     dict['diameter'] = diameter  # diameter of the telescope, in cm unit
#     dict['obscure'] = obscure
#     # dict['coll_area'] = 3.14159 / 4.0 * diameter * diameter * (1.0 - obscure)
#                             # effective area of the telescope, cm^2
#
#     # out_file = open(refdata + 'csst/telescope/config.json', "w")
#     # json.dump(dict, out_file, indent=2)
#     # out_file.close()
#
#     return dict


def get_instrument_config(diameter=200, obscure=0, readout_noise=5.5,
                          readout_xbin=1, readout_ybin=1, gain=1, dark=0.017):

    dic = dict()
    # user parameter
    dic['diameter'] = diameter  # diameter of the telescope, in cm unit
    dic['obscure'] = obscure
    dic['readout_xbin'] = readout_xbin
    dic['readout_ybin'] = readout_ybin
    dic['gain'] = gain                            # e/ADU
    dic['dark'] = dark                        # e/s/pix
    dic['readout_noise'] = readout_noise      # e/pix
    dic['QE'] = 1.0                            # e/pix
    dic['efficiency_file'] = 'IFU_throughput.dat'

    # hidden parameters
    dic['ccd_xsize'] = 1.755555            # A, delta_lambda_per_pixel
    dic['ccd_ysize'] = 0.1                 # arcsec, spatial axis
    dic['extract_aper_width'] = 2 * readout_ybin    # extract spectrum with aperture of 2 pixels
    dic['spaxel_xsize'] = 0.2              # arcsec
    dic['spaxel_ysize'] = dic['extract_aper_width'] * 0.1   # arcsec
    dic['fov_xsize'] = 6                 # arcsec
    dic['fov_ysize'] = 6                 # arcsec
    dic['wave_start'] = 3500
    dic['wave_end'] = 10000
    dic['wave_delta'] = dic['ccd_xsize'] * dic['readout_xbin']


    # out_file = open(refdata + "csst/ifs/config.json", "w")
    # json.dump(dict, out_file, indent=2)
    # out_file.close()

    return dic


def get_throughput(filename):

    """
    read IFS throughput file

    Returns:
        throughputwave: unit in Angstrom
        throughputvalue: unit in energy fraction or photon fraction ?

    """

    # throughput_file = os.path.join(os.getenv('SNPP_refdata'), 'csst', 'ifs', 'IFU_throughput.dat')
    # throughput = pd.read_csv(throughput_file,
    #                          sep='\s+', skiprows=1, header=None, names=['nm', 'q'])
    # lambdaq = throughput.nm.values * 10  # nm to A
    # qifs = throughput.q.values  # ; throughput of the whole system,
    # # ;assuming the total throughput cannot reach the theory value, 0.3 is the upper limit.
    # qifs[qifs >= 0.3] = 0.3
    # qinput = 1.0                    # the throughput of the telescope
    # qtot = qifs * qinput            # *qe ;qtot of CSST already includes the CCD efficiency


    throughput_file = os.path.join(refdata, 'csst', 'ifs', filename)
    cat = pd.read_csv(throughput_file,
                      comment='#', sep='\s+', header=None, names=['nm', 'q'])
    throughputwave = cat['nm'].values * 10  # nm to A
    throughputvalue = cat['q'].values       # energy fraction or photon fraction??

    return throughputwave, throughputvalue


def build_default_source():

    dict = {}
    dict['spectrum'] = {'name': 'SFgal_texp_FeH0_tau5_Ew10_AGN1.fits', 'redshift': 0.0, 'ebv': 0.0, 'lines':[]}
    dict['normalization'] = {'value': 17.7, 'unit':'mag/arcsec^2', 'band': 'sdss_g'}

    # out_file = open(refdata + "source/config.json", "w")
    # json.dump(dict, out_file, indent=2)
    # out_file.close()

    return dict



def build_default_calc():

    calc = {}
    # calc['telescope'] = get_telescope_config()
    calc['configuration'] = get_instrument_config()
    calc['source'] = build_default_source()
    calc['zodiacal_level'] = 'med'
    calc['earthshine_level'] = 'med'
    calc['repn'] = 20.
    calc['obst'] = 300.

    calc['targetsnr'] = 10      # only be used in calculate_type = 'limitmag',
                                # in this case, normalization value is invalid


    return calc





import os
import json
import pandas as pd
import numpy as np
import pdb

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
snpp_refdata = os.path.join(path, 'refdata/')


def get_instrument_config(diameter=200, obscure=0,
                          readout_xbin=1, readout_ybin=1, gain=1, dark=0.017,
                          readout_noise=5.5, throughput_file='IFU_throughput.dat'):

    dic = dict()
    # user parameter
    dic['diameter'] = diameter  # diameter of the telescope, in cm unit
    dic['obscure'] = obscure
    dic['readout_xbin'] = readout_xbin
    dic['readout_ybin'] = readout_ybin
    dic['gain'] = gain                            # e/ADU
    dic['dark'] = dark                         # e/s/pix
    dic['readout_noise'] = readout_noise       # e/pix
    dic['QE'] = 1.0                            # e/pix
    dic['efficiency_file'] = throughput_file

    # hidden parameters
    dic['ccd_xsize'] = 1.755555            # A, delta_lambda_per_pixel
    dic['ccd_ysize'] = 0.1                 # arcsec, spatial axis
    dic['extract_aper_width'] = 2 * dic['readout_ybin']   # extract spectrum with aperture of 2 pixels
    dic['spaxel_xsize'] = 0.2              # arcsec
    dic['spaxel_ysize'] = dic['extract_aper_width'] * 0.1   # arcsec
    dic['fov_xsize'] = 6                 # arcsec
    dic['fov_ysize'] = 6                 # arcsec
    dic['wave_start'] = 3500
    dic['wave_end'] = 10000
    dic['wave_delta'] = dic['ccd_xsize'] * dic['readout_xbin']


    # out_file = open(refdata + "csst/ifs/config.json", "w")
    # json.dump(dict, out_file, indent=2)
    # out_file.close()

    return dic




# def get_throughput():
#
#     """
#     read IFS throughput file
#
#     Returns:
#         throughputwave: unit in Angstrom
#         throughputvalue: unit in energy fraction or photon fraction ?
#
#     """
#
#     # throughput_file = os.path.join(os.getenv('SNPP_refdata'), 'csst', 'ifs', 'IFU_throughput.dat')
#     # throughput = pd.read_csv(throughput_file,
#     #                          sep='\s+', skiprows=1, header=None, names=['nm', 'q'])
#     # lambdaq = throughput.nm.values * 10  # nm to A
#     # qifs = throughput.q.values  # ; throughput of the whole system,
#     # # ;assuming the total throughput cannot reach the theory value, 0.3 is the upper limit.
#     # qifs[qifs >= 0.3] = 0.3
#     # qinput = 1.0                    # the throughput of the telescope
#     # qtot = qifs * qinput            # *qe ;qtot of CSST already includes the CCD efficiency
#
#
#     throughput_file = os.path.join(snpp_refdata, 'csst', 'ifs', 'IFU_throughput.dat')
#     cat = pd.read_csv(throughput_file,
#                       comment='#', sep='\s+', header=None, names=['nm', 'q'])
#     throughputwave = cat['nm'].values * 10  # nm to A
#     throughputvalue = cat['q'].values       # energe fraction or photon fraction??
#
#     return throughputwave, throughputvalue





def build_default_source(source_type='', shape_type='', normalization_type=''):

    dic = dict()

    # creat an point source or an extended source
    dic['source_type'] = source_type
    dic['x_offset'] = 0.0
    dic['y_offset'] = 0.0

    if source_type == 'point':

        dic['geometry'] = {'shape': 'PSF', 'FWHM': 0.2}

    elif source_type == 'extended':

        # shape_type = 'gaussian', 'sersic', 'box',
        # only valid for extended source.
        if shape_type == 'gaussian':

            dic['geometry'] = {'shape': 'gaussian', 'FWHM': 0.2}

        elif shape_type == 'sersic':

            dic['geometry'] = {'shape': 'sersic', 'PA': 45, 'b2a': 0.5,
                                'sersic_n': 1, 'Re_maj': 2}

        elif shape_type == 'box':

            dic['geometry'] = {'shape': 'box', 'PA': 45,
                                'xwidth': 5, 'ywidth': 5}

        else:

            raise ValueError('shape_type for extended source need to be '
                             '"gaussian", "sersic" or "box". ')

    else:

        raise ValueError('source_type need to be "point" or "extended". ')

    # normalized at certain wavelength or filter.
    dic['normalization_type'] = normalization_type

    if normalization_type == 'wavelength':

        dic['normalization'] = {'value': 17.7, 'unit': 'mag/arcsec^2', 'wavelength': 5500}

    elif normalization_type == 'filter':

        dic['normalization'] = {'value': 17.7, 'unit': 'mag/arcsec^2', 'band': 'SDSS-g'}

    else:

        raise ValueError('normalization_type need to be "wavelength" or "filter". ')


    # specify the spectrum template
    dic['spectrum'] = {'name': 'SFgal_texp_FeH0_tau5_Ew10_AGN1.fits',
                        'redshift': 0.0, 'ebv': 0.0}





    # out_file = open(snpp_refdata + "source/config.json", "w")
    # json.dump(dict, out_file, indent=2)
    # out_file.close()

    return dic


def build_default_scene(n_source=1, source_type=(), shape_type=(), normalization_type=()):

    scene = []

    for i in range(n_source):

        scene.append(build_default_source(source_type=source_type,
                                          shape_type=shape_type,
                                          normalization_type=normalization_type))

    return scene


def build_default_bkg():

    dic = dict()

    dic['zodiacal_level'] = 'median'
    dic['earth_shine'] = 'median'

    return dic



# def build_default_calc(n_source=1, source_type=(), normalization_type=()):
#
#     calc = dict()
#     calc['configuration'] = get_instrument_config()
#     calc['source'] = build_default_scene(n_source=n_source,
#                                          source_type=source_type,
#                                          normalization_type=normalization_type)
#     calc['background'] = build_default_bkg()
#     calc['repn'] = 20.
#     calc['obst'] = 300.
#
#     calc['targetsnr'] = 10      # only be used in calculate_type = 'limitmag',
#                                 # in this case, normalization value is invalid
#
#
#     return calc









import pdb

from setuptools import setup, find_packages
import re

package = "ifs_etc"

with open("README.md", "r") as fh:
    long_description = fh.read()


def find_version(package):
    version_file = open("src/" + package + "/__init__.py").read()
    rex = r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format('__version__')
    return re.search(rex, version_file).group(1)

setup(
    name=package,
    version=find_version(package),
    description='exposure time calculator',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='linlin',
    author_email='linlin@shao.ac.cn',
    packages=find_packages(where="src"),
    install_requires=['pandas==1.3.3',
                      'numpy==1.20.2',
                      'h5py==2.8.0',
                      'einops==0.3.2',
                      'matplotlib==3.0.2',
                      'astropy==4.2.1',
                      'scipy==1.1.0',
                      'extinction==0.4.0',
                      'commentjson==0.9.0'],
    package_dir={"": "src"},
    include_package_data=True,
    # exclude_package_data={"": ["README.md"]},
    python_requires='>=3',
)




import numpy as np
import pandas as pd
import os
import astropy.io.fits as fits
import astropy.units as u
from scipy.integrate import simps
from einops import repeat, rearrange
import extinction
import pdb

from astropy.modeling.models import Sersic2D, Box2D, Gaussian2D

path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
snpp_refdata = os.path.join(path, 'refdata/')


class read_template(object):

    r"""
    get spectrum template using the input parameters

    .. todo::
        enable to add emission lines


    Args:
        isource (dict, including 3 keys):
            name:
                the spectrum filename
            redshift:
                the redshift applied to the template
            ebv:
                the extinction applied to the template

    Attributes:

        wave (`numpy.ndarray`, unit in Angstrom):
            the wavelength of the spectrum, redshift applied.
        flux (`numpy.ndarray`, unit in erg/s/cm^2/A)
            the flux of the spectrum, redshift and extinction considered.

    """

    def __init__(self, isource):

        # self.source = isource
        # self.line_definitions = self.source['lines']

        # snpp_path = '/Users/linlin/Pro/snpp_v1_05/refdata/'
        # template_filename = snpp_refdata + 'sed/' + self.source['name']
        # filtera = snpp_path + 'normalization/filters/sdss_g0.par'
        # result = input_mag_model(self.source['normalization']['value'],
        #                          galtpl, filtera)  # scale the template to given brightness

        template_filename = isource['name']
        template_redshift = isource['redshift']
        template_ebv = isource['ebv']

        if template_filename.endswith('.fits'):

            hdu = fits.open(template_filename)
            template_wave = hdu[1].data['wavelength']   # A
            template_flux = hdu[1].data['flux'] * 1e-12         # erg/s/A/cm2

            # If the data is big endian, swap the byte order to make it little endian
            if template_wave.dtype.byteorder == '>':
                template_wave = template_wave.byteswap().newbyteorder()

        elif template_filename.endswith('.txt'):

            cat = pd.read_csv(template_filename, sep='\s+', header=None)    # separate by blank space
            template_wave = cat[0]      # unit should be in Angstrom
            template_flux = cat[1]      # unit should be in erg/s/cm^2/A

        else:
            raise ValueError("File name need to be ended with '.fits' or '.txt'. ")

        # extinction require the dtype should be double.
        template_wave = np.array(template_wave, dtype='float64')

        # extinction correction
        # assuming Calzetti-00 extinction law.
        r_v = 4.5
        extinction_mag = extinction.calzetti00(template_wave,
                                               template_ebv * r_v,
                                               r_v, unit='aa')
        template_flux = extinction.apply(extinction_mag, template_flux)

        # redshift correction
        template_wave = template_wave * (1 + template_redshift)
        template_flux = template_flux / (1 + template_redshift)

        self.wave = template_wave
        self.flux = template_flux


class Grid(object):

    """
    Generate the xy-grid of the IFU FOV under the instrument configuration.

    Args:
        config

    Attributes:

        dx:
            spaxel size in x-direction, unit as arcsec
        dy:
            spaxel size in y-direction, unit as arcsec
        nx:
            the number of elements in xarr
        ny:
            the number of elements in yarr
        spaxel_area:
            dx * dy, unit as arcsec^2
        xarr:
            x-array, unit as integer numbers of 0.2 arcsec
        yarr:
            y-array, unit as integer numbers of 0.2 arcsec
        x2d:
            2d array in x, unit as integer numbers of 0.2 arcsec
        y2d:
            2d array in y, unit as integer numbers of 0.2 arcsec
    """

    def __init__(self, config):

        self.config = config['configuration']
        self.dx = self.config['spaxel_xsize']
        self.dy = self.config['spaxel_ysize']
        self.nx = int(self.config['fov_xsize'] / self.dx)
        self.ny = int(self.config['fov_ysize'] / self.dy)
        self.spaxel_area = self.dx * self.dy

        # in arcsec
        # self.xarr = np.arange(0, self.config['fov_xsize'], self.dx)
        # self.yarr = np.arange(0, self.config['fov_ysize'], self.dy)
        # self.x2d, self.y2d = np.meshgrid(self.xarr, self.yarr)

        # in spaxel in 0.2 step
        self.xarr = np.arange(0, 30, self.dx / 0.2)
        self.yarr = np.arange(0, 30, self.dy / 0.2)
        self.x2d, self.y2d = np.meshgrid(self.xarr, self.yarr)


def get_wavearr(config):

    instr_par = config['configuration']
    wave_start = instr_par['wave_start']
    wave_end = instr_par['wave_end']
    wave_delta = instr_par['ccd_xsize'] * instr_par['readout_xbin']

    ccdspec_wave = np.arange(wave_start, wave_end, wave_delta)

    return ccdspec_wave


class read_filter(object):

    def __init__(self, filter_name=''):
        """

        Args:
            filter_name:
        """

        filter_file = os.path.join(snpp_refdata, 'normalization', 'filters', filter_name+'.par')
        band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
        self.wave = band[0].values  # A
        self.throughput = band[1].values  # vaccum_pass
        self.wavemin = self.wave[0]
        self.wavemax = self.wave[-1]

        # find the central wavelength, effective wavelength, and FWHM of the given filter
        filtermid = (self.wavemax - self.wavemin) * 0.5  # A, central wavelength
        dwave = self.wave[1:] - self.wave[:-1]
        self.waveeff = np.nansum(dwave * self.wave[1:] * self.throughput[1:]) / \
                       np.nansum(dwave * self.throughput[1:])
                                                                            # A, effective wavelength
        rmax = np.max(self.throughput)
        nnn = np.where(self.throughput > 0.5 * rmax)[0]
        self.FWHMmin = self.wave[nnn[0]]      # wave range at FWHM
        self.FWHMmax = self.wave[nnn[-1]]
        self.wavefwhm = self.FWHMmax - self.FWHMmin  # A, FWHM


def filter_mag(objwave, objflux, filterwave, filterthroughtput, output='mag'):

    """
    Get AB magnitude in a certain filter.
    
    Args:
        objwave: unit as A
        objflux: unit as erg/s/cm^2/A
        filterwave: unit as A
        filterthroughtput: unit as detector signal per photon (use vaccum_pass for space telescope,
                              otherwise select a given airmass)
                            
    Return:
        AB magnitude in this band
    
    """

    # resample the throughtput to objwave
    ind = (objwave >= np.min(filterwave)) & (objwave <= np.max(filterwave))
    wavetmp = objwave[ind]
    phot_frac = np.interp(wavetmp, filterwave, filterthroughtput)  # phot fraction (/s/A/cm^2?)
    # convert to energy fraction
    # E_frac = E0/hv * N_frac / E0 ~ N_frac/nu ~ N_frac * lambda
    energy_frac = phot_frac * wavetmp  # convert to energy fraction

    # convert the objflux to erg/s/cm^2/Hz
    c = 3e18    # A/s
    objflux_hz = objflux[ind] * (objwave[ind]**2) / c    # erg/s/scm^2/Hz

    from scipy.integrate import simps
    integrate_flux = simps(objflux_hz * energy_frac, c/wavetmp) / simps(energy_frac, c/wavetmp)

    if output == 'mag':
        mag = -2.5 * np.log10(integrate_flux) - 48.6
        return mag
    elif output == 'flux':
        return integrate_flux
    else:
        print('Error: output need to be "mag" or "flux". ')
        return np.nan


def normalized(template_wave, template_flux, config):
    """
    Scale the spectrum template to the given value.

    Args:
        template_wave:
            unit in A
        template_flux:
            unit in erg/s/cm^2/A
        config:
            the normalization dict: {'value', 'unit', 'band' or 'wavelength'}
            the geometry info to generate the 2D surface brightness map

    Returns:
        the scaled spectrum cube.

    """

    grid = Grid(config)

    source = config['source']
    n_source = len(source)
    cube_flux = 0

    for i in range(n_source):

        inormalize = source[i]['normalization']

        if 'band' in inormalize.keys():

            # normlized at certain filter
            filter = read_filter(inormalize['band'])
            normalized_wave = filter.waveeff
            ind_filter = (template_wave >= filter.wavemin) & (template_wave <= filter.wavemax)
            filter_wave = template_wave[ind_filter]
            filter_flux = np.interp(filter_wave, filter.wave, filter.throughput)
            filter_constant = simps(filter_flux * filter_wave, filter_wave)

            template_constant = simps(filter_flux * template_wave[ind_filter] * template_flux[ind_filter],
                                      template_wave[ind_filter])

            template_flux_norm = template_flux / (template_constant / filter_constant)

        else:

            # normlized at certain wavelength
            normalized_wave = inormalize['wavelength']
            template_flux0 = np.interp(normalized_wave, template_wave, template_flux)
            template_flux_norm = template_flux / template_flux0


        # get scaled factor at the normalized wave
        # convert the unit to erg/s/cm^2/A
        if inormalize['unit'] == 'erg/s/cm^2/Hz':
            u0 = (inormalize['value'] * u.erg / u.s / u.cm ** 2 / u.Hz).to(
                u.erg / u.s / u.cm ** 2 / u.AA, equivalencies=u.spectral_density(normalized_wave * u.AA)
            )
            u0 = u0.value
        elif inormalize['unit'] == 'erg/s/cm^2/A':
            u0 = inormalize['value']
        elif (inormalize['unit'] == 'AB magnitude') or (inormalize['unit'] == 'AB magnitude/arcsec^2'):
            u0 = (10**((inormalize['value'] + 48.6)/(-2.5)) * u.erg / u.s / u.cm ** 2 / u.Hz).to(
                u.erg / u.s / u.cm ** 2 / u.AA, equivalencies=u.spectral_density(normalized_wave * u.AA)
            )
            u0 = u0.value
        else:
            raise ValueError('Unrecognized unit of the source.')


        # get scaled 2d image
        # https://docs.astropy.org/en/stable/modeling/predef_models2D.html
        # input xy-offset unit as arcsec
        # input FWHM unit as arcsec
        # output xy-grid unit in spaxels (0.2" per grid)

        geometry = source[i]['geometry']
        xcen = grid.nx / 2 + source[i]['position']['x_offset'] / grid.dx
        ycen = grid.ny / 2 + source[i]['position']['y_offset'] / grid.dy

        if geometry['shape'] == 'sersic':

            rmaj = geometry['Re_maj'] / grid.dx
            rmin = geometry['b2a'] * rmaj
            sersicn = geometry['sersic_n']
            ellip = 1 - rmin/rmaj
            theta = (geometry['PA'] + 90) / 180 * np.pi
            u_re = u0

            mod = Sersic2D(amplitude=u_re, r_eff=rmaj,
                           n=sersicn, x_0=xcen, y_0=ycen,
                           ellip=ellip, theta=theta)

        elif geometry['shape'] == 'flatbox':

            xwidth = geometry['xwidth'] / grid.dx
            ywidth = geometry['ywidth'] / grid.dy

            mod = Box2D(amplitude=u0, x_0=xcen, y_0=ycen,
                        x_width=xwidth, y_width=ywidth)


        elif geometry['shape'] == 'gaussian':

            u_peak = u0
            x_stddev = geometry['x_fwhm'] / 2.35 / grid.dx
            y_stddev = geometry['y_fwhm'] / 2.35 / grid.dy
            theta = (geometry['PA'] + 90) / 180 * np.pi
            mod = Gaussian2D(amplitude=u_peak, x_mean=xcen, y_mean=ycen,
                        x_stddev=x_stddev, y_stddev=y_stddev,
                        theta=theta)

        elif geometry['shape'] == 'PSF':

            # only valid for point source, FWHM = 0.2", u0 is the total flux.
            # the normalized value is thought to be the integral value.
            # for gaussian integral flux = 2 * pi * A * sigma_x * sigma_y
            # grid.dx = 0.2"
            x_stddev = 0.2 / 2.35 / grid.dx
            y_stddev = 0.2 / 2.35 / grid.dy
            # surface brightness at the peak
            u_peak = u0 / 2 / np.pi / x_stddev / y_stddev / (grid.dx * grid.dy)
            mod = Gaussian2D(amplitude=u_peak, x_mean=xcen, y_mean=ycen,
                        x_stddev=x_stddev, y_stddev=y_stddev)

        else:
                print(geometry['shape'] + ' is not defined.')
                mod = None

        # generate the xy grid
        # unit in spaxel (0.2" per grid)
        scaled_img = mod(grid.x2d, grid.y2d)

        # make to a cube for each component
        tmp_template_flux = repeat(template_flux_norm, 'h -> h w c', w=grid.ny, c=grid.nx)
        tmp_factor = repeat(scaled_img, 'w c -> h w c', h=len(template_wave))
        norm_flux = tmp_template_flux * tmp_factor

        cube_flux = cube_flux + norm_flux


    # generate the wavelength cube
    cube_wave = repeat(template_wave, 'h -> h w c', w=grid.ny, c=grid.nx)


    # check the magnitude
    filter = read_filter('sdss_g')
    mag2d = np.zeros(shape=(grid.ny, grid.nx), dtype=float)
    for j in range(grid.ny):
        for i in range(grid.nx):
            mag2d[j, i] = filter_mag(cube_wave[:, j, i], cube_flux[:, j, i],
                                   filter.wave, filter.throughput, output='mag')


    return cube_wave, cube_flux, mag2d


class ModelCube(object):

    def __init__(self, config):
        # 生成的cube（波长和flux）对应了ccd上数据点（按config设置）

        self.ccdspec_wave = get_wavearr(config)
        self.ccdspec_nw = len(self.ccdspec_wave)

        if config['source_mode'] == 2:

            n_source = len(config['source'])
            for i in range(n_source):

                iobject = config['source'][i]

                template = read_template(iobject['spectrum'])
                template_wave_interp = self.ccdspec_wave
                template_flux_interp = np.interp(template_wave_interp, template.wave, template.flux)

                # self.ccdspec_flux = np.zeros(shape=(self.ccdspec_nw, grid.ny, grid.nx), dtype=np.float32)

                self.wavecube, self.fluxcube, self.mag2d = normalized(template_wave_interp, template_flux_interp,
                                                                      config)
                # 这部分循环有问题，这里n_source循环一次，normalized对n_source又循环一次
                # 不同template并没有起作用。
                # todo: 把normalized里的循环取消，在这里增加循环，以及流量叠加。

        elif config['source_mode'] == 1:

            import astropy.io.fits as fits
            dat = fits.open(config['source']['filename'])
            template_hdr = dat[1].header
            wave_start = template_hdr['CRVAL1'] * 1e10
            wave_delta = template_hdr['CDELT1'] * 1e10
            wave_end = wave_start + template_hdr['NAXIS1']*wave_delta
            template_wave = np.arange(wave_start, wave_end, wave_delta)
            template_flux = dat[1].data * 1e-17        # erg/s/cm^/A
            nx = template_hdr['NAXIS2']
            ny = template_hdr['NAXIS3']

            wavecube = repeat(template_wave, 'h -> h w c', w=ny, c=nx)
            wavecube_interp = repeat(self.ccdspec_wave, 'h -> h w c', w=ny, c=nx)

            from scipy.interpolate import interp1d
            fluxcube_reshape = rearrange(template_flux, 'h w c -> c h w')
            fluxcube_interp = wavecube_interp.copy()
            filter = read_filter('sdss_g')
            mag2d = np.zeros(shape=(ny, nx), dtype=float)
            for i in range(nx):
                for j in range(ny):
                    f = interp1d(wavecube[:, j, i], fluxcube_reshape[:, j, i], fill_value='extrapolate')
                    fluxcube_interp[:, j, i] = f(wavecube_interp[:, j, i])
                    mag2d[j, i] = filter_mag(wavecube_interp[:, j, i], fluxcube_interp[:, j, i],
                                             filter.wave, filter.throughput, output='mag')

            self.wavecube = wavecube_interp
            self.fluxcube = fluxcube_interp
            self.mag2d = mag2d


    # def export_to_fits(self, fitsfile='ModelCube'):


# class ConvolvedCube(object):

